% The Czech Republic %
@variables:transition

lgdp, lgdp_eq, lgdp_gap, dot_gdp, dot4_gdp, dot_gdp_eq 
lcpi, dot_cpi, dot4_cpi, rmc
ls, dot_s, prem, shock_prem
rn, rr, rr_eq, rr_gap, rn_neutral
dot_z, lz_gap, dot_z_eq, lz, lz_eq
mci
target
lx_gdp_gap, x_rn, x_rr, x_rr_gap, x_rr_eq, dot_x_cpi, lx_cpi
ls_tar, dot_s_tar
cum_gap

% ------------------------------------------------------------------------------- %
@variables:innovation

shk_lgdp_gap, shk_dot_cpi, shk_ls, shk_rn, shk_dot_s_tar, shk_target
shk_lx_gdp_gap, shk_x_rn, shk_dot_x_cpi, shk_x_rr_eq
shk_rr_eq, shk_dot_z_eq, shk_prem, shk_dot_gdp_eq

% ------------------------------------------------------------------------------- %
@parameters

a1, a2, a3, a4
b1, b2, b3
e1
f1, f2, f3
g1
h0, h1, h2, h3
t1

target_ss
ss_rr_eq, ss_dot_z_eq, ss_dot_gdp_eq
ss_dot_x_cpi, ss_x_rr_eq

% ------------------------------------------------------------------------------- %
@equations:transition

%% Aggregate demand
lgdp_gap = a1*lgdp_gap{-1} - a2*mci + a3*lx_gdp_gap + shk_lgdp_gap;

%% Real monetary condition index
mci = a4*rr_gap + (1-a4)*(- lz_gap);

%% Aggregate supply (the Phillips curve)
dot_cpi =  b1*dot_cpi{-1} + (1-b1)*dot_cpi{+1} + b2*rmc + shk_dot_cpi;

%% Real marginal cost
rmc = b3*lgdp_gap + (1-b3)*lz_gap;

%% Uncovered Interest Rate Parity (UIP) 
% A. UIP with a backward-looking element ("persistent" exchange rate)
ls = (1-e1)*ls{+1} + e1*(ls{-1} + 2/4*(target - ss_dot_x_cpi + dot_z_eq)) + (- rn + x_rn + prem)/4 + shk_ls;

%% B. UIP when the exchange rate is managed to meet the inflation objective
%ls = e1*ls_tar + (1-e1)*(ls{+1} + (x_rn - rn + prem)/4 + shk_ls);

%% Monetary policy reaction function (a forward-looking Taylor rule)
%% A. The standard rule
rn = f1*rn{-1} + (1-f1)*(rn_neutral + f2*(dot4_cpi{+4} - target{+4}) + f3*lgdp_gap) + shk_rn;

%% B. The rule modified for imperfect control over the domestic money market
%rn = g1*(4*(ls{+1} - ls) + x_rn + prem) + (1-g1)*(f1*rn{-1} + (1-f1)*(rn_neutral + f2*(dot4_cpi{+4} - target{+4}) + f3*lgdp_gap)) + shk_rn;

%% Identities & definitions
dot_s_tar = t1*dot_s_tar{-1} + (1-t1)*(target - ss_dot_x_cpi + dot_z_eq) + shk_dot_s_tar;
ls_tar = ls_tar{-1} + dot_s_tar/4;

target = t1*target{-1} + (1-t1)*target_ss + shk_target;

rn_neutral = rr_eq + dot_cpi{+1};

rr = rn - dot_cpi{+1};
rr_gap = rr - rr_eq;

lz = ls + lx_cpi - lcpi;
lz_gap = lz - lz_eq;

dot_z = 4*(lz - lz{-1}); 

lgdp_gap = lgdp - lgdp_eq;
lgdp_eq = lgdp_eq{-1} + dot_gdp_eq/4;
dot_gdp = 4*(lgdp - lgdp{-1});
dot4_gdp = lgdp - lgdp{-4};

lcpi = lcpi{-1} + dot_cpi/4;
dot4_cpi = (dot_cpi + dot_cpi{-1} + dot_cpi{-2} + dot_cpi{-3})/4;

ls = ls{-1} + dot_s/4;

%% Trends
dot_z_eq{+1} = rr_eq - x_rr_eq - prem + shock_prem;
shock_prem = h0*shock_prem{-1} + shk_prem;

dot_z_eq = h1*dot_z_eq{-1} + (1-h1)*ss_dot_z_eq + shk_dot_z_eq;
lz_eq = lz_eq{-1} + dot_z_eq/4;

rr_eq = h1*rr_eq{-1} + (1-h1)*ss_rr_eq + shk_rr_eq;

dot_gdp_eq = h1*dot_gdp_eq{-1} + (1-h1)*ss_dot_gdp_eq + shk_dot_gdp_eq;

%% Exogenous equations
lx_gdp_gap = h2*lx_gdp_gap{-1} + shk_lx_gdp_gap;
x_rn = h3*x_rn{-1} + (1-h3)*(x_rr_eq + dot_x_cpi) + shk_x_rn;
x_rr = x_rn - dot_x_cpi;
x_rr_eq = h1*x_rr_eq{-1} + (1-h1)*ss_x_rr_eq + shk_x_rr_eq;
x_rr_gap = x_rr - x_rr_eq;
dot_x_cpi = h1*dot_x_cpi{-1} + (1-h1)*ss_dot_x_cpi + shk_dot_x_cpi;
lx_cpi = lx_cpi{-1} + dot_x_cpi/4;

%% Cummulative gap
cum_gap = cum_gap{-1} + lgdp_gap;

@if filter = true
@variables:measurement
L_GDP, L_S, L_CPI, RN
L_X_CPI, X_RN

@equations:measurement
L_GDP = lgdp;
L_S = ls;
L_CPI = lcpi;
RN = rn;
L_X_CPI = lx_cpi;
X_RN = x_rn;

@stop

Legend
lcpi       log of price level
rn         nominal interest rate
rr         real interest rate
rn_neutral policy neutral rate
ls         log of nominal exchange rate
lz         log of real exchange rate
lgdp       log of gdp
mci        real monetary conditions index
rmc        real marginal cost 
_gap       cyclical deviation from a trend
_eq        trend (equilibrium)
ss_        steady-state value  
dot_       q-o-q change
dot4_      y-o-y change  
x_         foreign variable
shk_       equation residual